package cz.vutbr.fit.gja.androidspring;
import org.springframework.core.io.ClassPathResource;
import org.springframework.core.io.Resource;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;

import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.Toast;

public class HttpPostFormDataActivity extends AbstractAsyncActivity {

  protected static final String TAG = HttpPostFormDataActivity.class.getSimpleName();

  // ***************************************
  // Activity methods
  // ***************************************
  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    this.setContentView(R.layout.http_post_form_data_layout);

    // Initiate the POST request when the button is selected
    final Button button = (Button) findViewById(R.id.button_post_form_data);
    button.setOnClickListener(new View.OnClickListener() {
      public void onClick(View v) {
        new PostMessageTask().execute();
      }
    });
  }

  // ***************************************
  // Private methods
  // ***************************************
  private void showResult(String result) {
    if (result != null) {
      // display a notification to the user with the response information
      Toast.makeText(this, result, Toast.LENGTH_LONG).show();
    } else {
      Toast.makeText(this, "I got null, something happened!", Toast.LENGTH_LONG).show();
    }
  }

  // ***************************************
  // Private classes
  // ***************************************
  private class PostMessageTask extends AsyncTask<Void, Void, String> {

    private MultiValueMap<String, Object> formData;

    @Override
    protected void onPreExecute() {
      showLoadingProgressDialog();

      Resource resource = new ClassPathResource("res/drawable/spring09_logo.png");

      // populate the data to post
      formData = new LinkedMultiValueMap<String, Object>();
      formData.add("description", "Spring logo");
      formData.add("file", resource);
    }

    @Override
    protected String doInBackground(Void... params) {
      try {
        // The URL for making the POST request
        final String url = getString(R.string.base_uri) + "/postformdata";

        HttpHeaders requestHeaders = new HttpHeaders();

        // Sending multipart/form-data
        requestHeaders.setContentType(MediaType.MULTIPART_FORM_DATA);

        // Populate the MultiValueMap being serialized and headers in an HttpEntity object to use for the request
        HttpEntity<MultiValueMap<String, Object>> requestEntity = new HttpEntity<MultiValueMap<String, Object>>(
            formData, requestHeaders);

        // Create a new RestTemplate instance
        RestTemplate restTemplate = new RestTemplate(true);

        // Make the network request, posting the message, expecting a String in response from the server
        ResponseEntity<String> response = restTemplate.exchange(url, HttpMethod.POST, requestEntity,
            String.class);

        // Return the response body to display to the user
        return response.getBody();
      } catch (Exception e) {
        Log.e(TAG, e.getMessage(), e);
      }

      return null;
    }

    @Override
    protected void onPostExecute(String result) {
      dismissProgressDialog();
      showResult(result);
    }

  }

}
