package cz.vutbr.fit.gja.androidspring;

import java.util.Collections;

import org.springframework.http.HttpAuthentication;
import org.springframework.http.HttpBasicAuthentication;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.http.converter.json.MappingJacksonHttpMessageConverter;
import org.springframework.web.client.HttpClientErrorException;
import org.springframework.web.client.ResourceAccessException;
import org.springframework.web.client.RestTemplate;

import android.content.pm.PackageManager;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;
import androidx.core.app.ActivityCompat;

public class MainActivity extends AbstractAsyncActivity {

  protected static final String TAG = MainActivity.class.getSimpleName();

  // ***************************************
  // Activity methods
  // ***************************************
  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.activity_main);

    final int N_REQUEST_CODE = 1;
    if (!(checkSelfPermission(
            android.Manifest.permission.ACCESS_NETWORK_STATE) == PackageManager.PERMISSION_GRANTED)) {
      ActivityCompat.requestPermissions(this, new String[]{android.Manifest.permission.ACCESS_NETWORK_STATE}, N_REQUEST_CODE);
    }
    final int I_REQUEST_CODE = 2;
    if (!(checkSelfPermission(
            android.Manifest.permission.INTERNET) == PackageManager.PERMISSION_GRANTED)) {
      ActivityCompat.requestPermissions(this, new String[]{android.Manifest.permission.INTERNET}, I_REQUEST_CODE);
    }

    // Initiate the request to the protected service
    final Button submitButton = (Button) findViewById(R.id.submit);
    submitButton.setOnClickListener(new View.OnClickListener() {
      public void onClick(View v) {
        new FetchSecuredResourceTask().execute();
      }
    });
  }
  
  // ***************************************
  // Private methods
  // ***************************************
  private void displayResponse(Message response) {
    Toast.makeText(this, response.getText(), Toast.LENGTH_LONG).show();
  }

  // ***************************************
  // Private classes
  // ***************************************
  private class FetchSecuredResourceTask extends AsyncTask<Void, Void, Message> {

    private String username;

    private String password;

    @Override
    protected void onPreExecute() {
      showLoadingProgressDialog();

      // build the message object
      EditText editText = (EditText) findViewById(R.id.username);
      this.username = editText.getText().toString();

      editText = (EditText) findViewById(R.id.password);
      this.password = editText.getText().toString();
    }

    @Override
    protected Message doInBackground(Void... params) {
      final String url = "http://10.0.2.2:8080";

      // Populate the HTTP Basic Authentitcation header with the username and password
      HttpAuthentication authHeader = new HttpBasicAuthentication(username, password);
      HttpHeaders requestHeaders = new HttpHeaders();
      requestHeaders.setAuthorization(authHeader);
      requestHeaders.setAccept(Collections.singletonList(MediaType.APPLICATION_JSON));

      // Create a new RestTemplate instance
      RestTemplate restTemplate = new RestTemplate();
      restTemplate.getMessageConverters().add(new MappingJacksonHttpMessageConverter());

      try {
        // Make the network request
        Log.d(TAG, url);
        ResponseEntity<Message> response = restTemplate.exchange(url, HttpMethod.GET, new HttpEntity<Object>(requestHeaders), Message.class);
        return response.getBody();
      } catch (HttpClientErrorException e) {
        Log.e(TAG, e.getLocalizedMessage(), e);
        return new Message(0, e.getStatusText(), e.getLocalizedMessage());
      } catch (ResourceAccessException e) {
        Log.e(TAG, e.getLocalizedMessage(), e);
        return new Message(0, e.getClass().getSimpleName(), e.getLocalizedMessage());
      }
    }

    @Override
    protected void onPostExecute(Message result) {
      dismissProgressDialog();
      displayResponse(result);
    }

  }
  
}

